/*
  Java gfx-test framework

   Copyright (C) 2013, 2014  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
*/

package org.gfxtest.testsuites;

import java.awt.Graphics2D;
import java.awt.geom.Area;



import org.gfxtest.framework.CommonCAGOperations;
import org.gfxtest.framework.CommonRenderingStyles;
import org.gfxtest.framework.GfxTest;
import org.gfxtest.framework.TestImage;
import org.gfxtest.framework.TestResult;
import org.gfxtest.framework.annotations.GraphicsPrimitive;
import org.gfxtest.framework.annotations.GraphicsPrimitives;
import org.gfxtest.framework.annotations.RenderStyle;
import org.gfxtest.framework.annotations.RenderStyles;
import org.gfxtest.framework.annotations.TestType;
import org.gfxtest.framework.annotations.TestTypes;
import org.gfxtest.framework.annotations.Transformation;
import org.gfxtest.framework.annotations.Transformations;
import org.gfxtest.framework.annotations.Zoom;



/**
 * This test checks the process of creating and rendering new geometric shapes
 * by performing boolean operations on existing ones. This process is often
 * called Constructive area geometry (CAG). Various cases are checked by this
 * test suite.
 * 
 * @author Pavel Tisnovsky
 */
@TestType(TestTypes.RENDER_TEST)
@GraphicsPrimitive(GraphicsPrimitives.AREA)
@RenderStyle(RenderStyles.FILL)
@Transformation(Transformations.NONE)
@Zoom(1)
public class CAGOperationsOnRoundRectangleAndRectangle extends GfxTest
{

    /**
     * Checks the process of creating and rendering new geometric shape
     * constructed from two rectangles using union operator. The shape is
     * rendered using stroke.
     * 
     * @param image
     *            image to which area is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testUnionStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // create area using union operator
        Area area = CommonCAGOperations.createAreaFromRoundRectangleAndRectangleUsingUnionOperator(image);
        // draw the area
        graphics2d.draw(area);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Checks the process of creating and rendering new geometric shape
     * constructed from two rectangles using subtract operator. The shape is
     * rendered using stroke.
     * 
     * @param image
     *            image to which area is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testSubtractStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // create area using union operator
        Area area = CommonCAGOperations.createAreaFromRoundRectangleAndRectangleUsingSubtractOperator(image);
        // draw the area
        graphics2d.draw(area);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Checks the process of creating and rendering new geometric shape
     * constructed from two rectangles using inverse subtract operator. The shape is
     * rendered using stroke.
     * 
     * @param image
     *            image to which area is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testInverseSubtractStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // create area using union operator
        Area area = CommonCAGOperations.createAreaFromRoundRectangleAndRectangleUsingInverseSubtractOperator(image);
        // draw the area
        graphics2d.draw(area);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Checks the process of creating and rendering new geometric shape
     * constructed from two rectangles using intersect operator. The shape is
     * rendered using stroke.
     * 
     * @param image
     *            image to which area is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIntersectStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // create area using union operator
        Area area = CommonCAGOperations.createAreaFromRoundRectangleAndRectangleUsingIntersectOperator(image);
        // draw the area
        graphics2d.draw(area);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Checks the process of creating and rendering new geometric shape
     * constructed from two rectangles using exclusive or operator. The shape is
     * rendered using stroke.
     * 
     * @param image
     *            image to which area is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testXorStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // create area using union operator
        Area area = CommonCAGOperations.createAreaFromRoundRectangleAndRectangleUsingXorOperator(image);
        // draw the area
        graphics2d.draw(area);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Checks the process of creating and rendering new geometric shape
     * constructed from two rectangles using union operator. The shape is
     * rendered using wide stroke.
     * 
     * @param image
     *            image to which area is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testUnionWideStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // create area using union operator
        Area area = CommonCAGOperations.createAreaFromRoundRectangleAndRectangleUsingUnionOperator(image);
        // draw the area
        graphics2d.draw(area);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Checks the process of creating and rendering new geometric shape
     * constructed from two rectangles using subtract operator. The shape is
     * rendered using wide stroke.
     * 
     * @param image
     *            image to which area is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testSubtractWideStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // create area using union operator
        Area area = CommonCAGOperations.createAreaFromRoundRectangleAndRectangleUsingSubtractOperator(image);
        // draw the area
        graphics2d.draw(area);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Checks the process of creating and rendering new geometric shape
     * constructed from two rectangles using inverse subtract operator. The shape is
     * rendered using wide stroke.
     * 
     * @param image
     *            image to which area is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testInverseSubtractWideStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // create area using union operator
        Area area = CommonCAGOperations.createAreaFromRoundRectangleAndRectangleUsingInverseSubtractOperator(image);
        // draw the area
        graphics2d.draw(area);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Checks the process of creating and rendering new geometric shape
     * constructed from two rectangles using intersect operator. The shape is
     * rendered using wide stroke.
     * 
     * @param image
     *            image to which area is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIntersectWideStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // create area using union operator
        Area area = CommonCAGOperations.createAreaFromRoundRectangleAndRectangleUsingIntersectOperator(image);
        // draw the area
        graphics2d.draw(area);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Checks the process of creating and rendering new geometric shape
     * constructed from two rectangles using exclusive or operator. The shape is
     * rendered using wide stroke.
     * 
     * @param image
     *            image to which area is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testXorWideStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeThickWidth(graphics2d);
        // create area using union operator
        Area area = CommonCAGOperations.createAreaFromRoundRectangleAndRectangleUsingXorOperator(image);
        // draw the area
        graphics2d.draw(area);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Checks the process of creating and rendering new geometric shape
     * constructed from two rectangles using union operator. The shape is
     * rendered using extra wide stroke.
     * 
     * @param image
     *            image to which area is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testUnionExtraWideStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // create area using union operator
        Area area = CommonCAGOperations.createAreaFromRoundRectangleAndRectangleUsingUnionOperator(image);
        // draw the area
        graphics2d.draw(area);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Checks the process of creating and rendering new geometric shape
     * constructed from two rectangles using subtract operator. The shape is
     * rendered using extra wide stroke.
     * 
     * @param image
     *            image to which area is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testSubtractExtraWideStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // create area using union operator
        Area area = CommonCAGOperations.createAreaFromRoundRectangleAndRectangleUsingSubtractOperator(image);
        // draw the area
        graphics2d.draw(area);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Checks the process of creating and rendering new geometric shape
     * constructed from two rectangles using inverse subtract operator. The shape is
     * rendered using extra wide stroke.
     * 
     * @param image
     *            image to which area is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testInverseSubtractExtraWideStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // create area using union operator
        Area area = CommonCAGOperations.createAreaFromRoundRectangleAndRectangleUsingInverseSubtractOperator(image);
        // draw the area
        graphics2d.draw(area);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Checks the process of creating and rendering new geometric shape
     * constructed from two rectangles using intersect operator. The shape is
     * rendered using extra wide stroke.
     * 
     * @param image
     *            image to which area is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testIntersectExtraWideStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // create area using union operator
        Area area = CommonCAGOperations.createAreaFromRoundRectangleAndRectangleUsingIntersectOperator(image);
        // draw the area
        graphics2d.draw(area);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Checks the process of creating and rendering new geometric shape
     * constructed from two rectangles using exclusive or operator. The shape is
     * rendered using extra wide stroke.
     * 
     * @param image
     *            image to which area is to be drawn
     * @param graphics2d
     *            graphics canvas
     * @return test result status - PASSED, FAILED or ERROR
     */
    public TestResult testXorExtraWideStrokePaint(TestImage image, Graphics2D graphics2d)
    {
        // set stroke color
        CommonRenderingStyles.setStrokeColor(graphics2d);
        // set stroke width
        CommonRenderingStyles.setStrokeExtraThickWidth(graphics2d);
        // create area using union operator
        Area area = CommonCAGOperations.createAreaFromRoundRectangleAndRectangleUsingXorOperator(image);
        // draw the area
        graphics2d.draw(area);
        // test result
        return TestResult.PASSED;
    }

    /**
     * Entry point to the test suite.
     *
     * @param args not used in this case
     */
    public static void main(String[] args)
    {
        new CAGOperationsOnRoundRectangleAndRectangle().runTestSuite(args);
    }
}
